@@ -4,169 +4,93 @@
 #' @description Plot an approximation of the posterior density.
 #' @param beta_posterior_density a list. The result of the function
 #'                 \code{compute_beta_posterior_density}.
-#' @param param a list containing: (optional)
-#' \describe{
-#' \item{cols}{a vector of colors for the function image.}
-#' \item{main}{an overall title for the plot.}
-#' \item{xlab}{a title for the x axis.}
-#' \item{ylab}{a title for the y axis.}
-#' \item{ylim}{a numeric vectors of length 2, giving the y coordinate range.}
-#' }
+#' @param param an optional  list containing arguments: col_low, col_mid, col_high,
+#'          ylim, xlab, ylab, title.
 #' @param q an integer (optional), the index of the functional covariate to plot.
-#' @importFrom stats quantile
-#' @importFrom grDevices heat.colors
+#' @param to_print display the plot if TRUE.
 #' @export
 #' @examples
-#' library(RColorBrewer)
 #' data(data1)
 #' data(param1)
 #' data(res_bliss1)
-#' param1$cols <- colorRampPalette(brewer.pal(9,"Reds"))(1e2)
-#' image_Bliss(res_bliss1$beta_posterior_density,param1,q=1)
-#' lines(res_bliss1$data$grids[[1]],res_bliss1$Bliss_estimate[[1]],type="s",lwd=2)
-#' lines(res_bliss1$data$grids[[1]],res_bliss1$data$betas[[1]],col=3,lwd=2,type="s")
 #'
-#' # ---- not run
-#' param1$cols <- colorRampPalette(brewer.pal(9,"YlOrRd"))(1e2)
 #' image_Bliss(res_bliss1$beta_posterior_density,param1,q=1)
-#' lines(res_bliss1$data$grids[[1]],res_bliss1$Bliss_estimate[[1]],type="s",lwd=2)
-#' lines(res_bliss1$data$grids[[1]],res_bliss1$data$betas[[1]],col=3,lwd=2,type="s")
-#'
-#' param1$cols <- rev(heat.colors(12))
-#' param1$col_scale <- "quantile"
-#' image_Bliss(res_bliss1$beta_posterior_density,param1,q=1)
-#' lines(res_bliss1$data$grids[[1]],res_bliss1$Bliss_estimate[[1]],type="s",lwd=2)
-#' lines(res_bliss1$data$grids[[1]],res_bliss1$data$betas[[1]],col=3,lwd=2,type="s")
-#'
-#' param1$cols <- rev(terrain.colors(12))
-#' image_Bliss(res_bliss1$beta_posterior_density,param1,q=1)
-#' lines(res_bliss1$data$grids[[1]],res_bliss1$Bliss_estimate[[1]],type="s",lwd=2)
-#' lines(res_bliss1$data$grids[[1]],res_bliss1$data$betas[[1]],col=2,lwd=2,type="s")
-#'
-#' param1$cols <- rev(topo.colors(12))
-#' image_Bliss(res_bliss1$beta_posterior_density,param1,q=1)
-#' lines(res_bliss1$data$grids[[1]],res_bliss1$Bliss_estimate[[1]],type="s",lwd=2)
-#' lines(res_bliss1$data$grids[[1]],res_bliss1$data$betas[[1]],col=2,lwd=2,type="s")
-image_Bliss <- function(beta_posterior_density,param=list(),q=1){
-  if(length(param) != 0){ # PMG 2018-11-13
-    cols      <- param[["cols"]] #Ceci n'est pas une modification pmg 08-03-18
-    main      <- param[["main"]]
-    ylab      <- param[["ylab"]]
-    xlab      <- param[["xlab"]]
-    ylim      <- param[["ylim"]]
-  }else{
-    cols      <- NULL
-    main      <- NULL
-    ylab      <- NULL
-    xlab      <- NULL
-    ylim      <- NULL
-  }
- if(is.null(cols)){
-   cols <- rev(heat.colors(100))
- }
- if(is.null(main)){
-   main <- ""
- }
-  if(is.null(ylab)){
-    ylab <- ""
-  }
-  if(is.null(xlab)){
-    xlab <- ""
-  }
-  if(is.null(ylim)){
-    ylim  <- range(ylim , beta_posterior_density[[q]]$grid_beta_t)
-  }
+image_Bliss <- function(beta_posterior_density,param=list(),q=1,to_print=TRUE){
+  ########## Initialization - Load objects
+  col_low <- param[["col_low"]]
+  col_mid <- param[["col_mid"]]
+  col_high <- param[["col_high"]]
 
- image(beta_posterior_density[[q]]$grid_t,
-       beta_posterior_density[[q]]$grid_beta_t,
-       beta_posterior_density[[q]]$density,
-       col=cols,main=main,xlab=xlab,ylab=ylab,ylim=ylim)
+  if(is.null(col_low)) col_low <- "white"
+  if(is.null(col_mid)) col_mid <- "yellow"
+  if(is.null(col_high)) col_high <- "red"
 
- # Fix problems about the axis (PMG 2018-11-11)
- x_tmp <- max(abs(beta_posterior_density[[q]]$grid_t))
- y_tmp <- max(abs(beta_posterior_density[[q]]$grid_beta_t))
- axis(1,at=ylim+c(-x_tmp,x_tmp) )
- axis(2,at=ylim+c(-y_tmp,y_tmp) )
- axis(3,at=ylim+c(-x_tmp,x_tmp) )
- axis(4,at=ylim+c(-y_tmp,y_tmp) )
-}
+  grid_t <- beta_posterior_density[[q]][["grid_t"]]
+  grid_beta_t <- beta_posterior_density[[q]][["grid_beta_t"]]
+  density <- beta_posterior_density[[q]][["density"]]
 
-################################# ----
-#' plot_bliss
-################################# ----
-#' @description A suitable representation of the Bliss estimate.
-#' @param x the coordinates of points in the plot.
-#' @param y the y coordinates of points in the plot.
-#' @param connect a logical value (optional), to handle discontinuous function.
-#'        If \code{connect} is TRUE, the plot is one line. Otherwise, several
-#'        lines are used.
-#' @param xlab a title for the x axis.
-#' @param ylab a title for the y axis.
-#' @param ylim a numeric vectors of length 2, giving the y coordinate range.
-#' @param ... Arguments to be passed to methods, such as graphical parameters
-#'        (see \code{par}).
-#' @importFrom grDevices gray.colors
-#' @importFrom graphics abline segments axis image lines matplot plot text
-#' @export
-#' @examples
-#' \donttest{
-#' data(data1)
-#' data(param1)
-#' # res_bliss1 <- fit_Bliss(data=data1,param=param1,verbose=TRUE)
-#' }
-#' data(res_bliss1)
-#' ### Plot the BLiss estimate on a suitable grid
-#' plot_bliss(res_bliss1$data$grids[[1]],
-#'            res_bliss1$Bliss_estimate[[1]],lwd=2,bound=FALSE)
-plot_bliss <- function(x,y,connect=FALSE,xlab="",ylab="",ylim=NULL,...){
- if(is.null(ylim)) ylim <- range(y)
- plot(x,x,type="n",ylim=ylim,ylab=ylab,xlab=xlab,...)
- lines_bliss(x,y,connect=connect,...)
+  ########## Pretreatment
+  df_density <- expand.grid(grid_t,grid_beta_t)
+  names(df_density) <- c("grid_t","grid_beta_t")
+
+  df_density$density = as.vector(density)
+  density_max <- max(df_density$density)
+
+  ########## Do the plot
+  p <- ggplot2::ggplot(df_density) + ggplot2::aes(x=grid_t,y=grid_beta_t) +
+    ggplot2::geom_tile(ggplot2::aes(fill=density)) +
+    ggplot2::scale_fill_gradient2(low = col_low,mid = col_mid,high = col_high,
+                         midpoint = density_max/2,guide = "none") +
+    ggplot2::xlab("") + ggplot2::ylab("") +
+    ggplot2::theme(panel.background = ggplot2::element_blank()) # theme_classic()
+
+  ########## Add options
+  if(!is.null(param[["ylim"]]))
+    p <- p + ggplot2::ylim(param[["ylim"]][1],param[["ylim"]][2])
+  if(!is.null(param[["xlab"]]))
+    p <- p + ggplot2::xlab(param[["xlab"]])
+  if(!is.null(param[["ylab"]]))
+    p <- p + ggplot2::ylab(param[["ylab"]])
+  if(!is.null(param[["title"]]))
+    p <- p + graphics::title(param[["title"]])
+
+  ########## Output
+  if(to_print) print(p)
+
+  ########## Output
+  return(p)
 }
 
 ################################# ----
 #' lines_bliss
 ################################# ----
-#' @description A suitable representation of the Bliss estimate.
+#' @description Add a line to a plot obtained with \code{image_Bliss}.
 #' @param x the coordinates of points in the plot.
 #' @param y the y coordinates of points in the plot.
-#' @param connect a logical value (optional), to handle discontinuous function.
-#'        If \code{connect} is TRUE, the plot is one line. Otherwise, several
-#'        lines are used.
-#' @param ... Arguments to be passed to methods, such as graphical parameters
-#'        (see \code{par}).
+#' @param col a color.
+#' @param lty option corresponding to "linetype" of \code{geom_line}.
 #' @export
 #' @examples
-#' ### Plot the BLiss estimate on a suitable grid
-#' \donttest{
 #' data(data1)
 #' data(param1)
-#' # res_bliss1 <- fit_Bliss(data=data1,param=param1,verbose=TRUE)
-#' }
 #' data(res_bliss1)
-#' ### Plot the BLiss estimate on a suitable grid
-#' plot_bliss(res_bliss1$data$grids[[1]],
-#'            res_bliss1$Bliss_estimate[[1]],lwd=2,bound=FALSE)
-#' lines_bliss(res_bliss1$data$grids[[1]],
-#'             res_bliss1$Smooth_estimate[[1]],lty=2)
-lines_bliss <- function(x,y,connect=FALSE,...){
-  # Compute a more 'interpretable grid'
-  extended_grid <- x - 0.5*diff(x)[1]
-  extended_grid <- c(extended_grid,max(extended_grid) + diff(extended_grid)[1]) # PMG 11-11-18
+#'
+#' image_Bliss(res_bliss1$beta_posterior_density,param1,q=1) +
+#' lines_bliss(res_bliss1$data$grids[[1]],res_bliss1$smooth_estimate[[1]])+
+#' lines_bliss(res_bliss1$data$grids[[1]],res_bliss1$Bliss_estimate[[1]],col="purple")
+#'
+lines_bliss <- function(x,y,col="black",lty="solid"){
+  ########## Initialization - Pretreatment
+  df <- data.frame(x=as.vector(x),y=as.vector(y))
 
- for(i in 1:length(extended_grid)){
-  segments(extended_grid[i],y[i],
-           extended_grid[i+1],y[i],
-           ...)
-  if(connect & i > 1)
-   segments(extended_grid[i],y[i],
-            extended_grid[i],y[i-1],
-            ...)
- }
+  ########## Do the plot
+  word <- ggplot2::geom_line(data=df,ggplot2::aes(x=x,y=y),col=col,linetype=lty)
+
+  ########## Output
+  return(word)
 }
 
 
-
 ################################# ----
 #' interpretation_plot
 ################################# ----
@@ -199,8 +123,8 @@ interpretation_plot <- function(data,Bliss_estimate,q=1,centered=FALSE,cols=NULL
   grid  <- data$grids[[q]]
 
   # Graphical options
-  if(is.null(cols)) cols  <- rev(heat.colors(length(y)))
-  bg_col  <- rev(gray.colors(length(y)))
+  if(is.null(cols)) cols  <- rev(grDevices::heat.colors(length(y)))
+  bg_col  <- rev(grDevices::gray.colors(length(y)))
   grid_y <- seq(min(y),max(y),length=length(y))
   match  <- sapply(y,function(v) order(abs(v - grid_y))[1])
   cols   <- cols[match]
@@ -237,48 +161,48 @@ interpretation_plot <- function(data,Bliss_estimate,q=1,centered=FALSE,cols=NULL
     for(i in seq(3,ncol(new_scaled_x)-1,by=2))
       new_scaled_x[,i] <- 0.5*(new_scaled_x[,i-1]+new_scaled_x[,i+1])
 
-    matplot(grid,t(scaled_x) ,type="l",lty=1,col=cols,lwd=lwds+1,
+    graphics::matplot(grid,t(scaled_x) ,type="l",lty=1,col=cols,lwd=lwds+1,
             main="",xlab="",ylab="")
     for(i in 1:nrow(intervals)){
-      if(intervals[i,3]!=0) text( (extended_grid[intervals[i,1]] + grid[intervals[i,2]])/2 ,
+      if(intervals[i,3]!=0) graphics::text( (extended_grid[intervals[i,1]] + grid[intervals[i,2]])/2 ,
                                   max(scaled_x), round(intervals[i,3],1),cex=1)
       if(intervals[i,3] == 0) index <- c(index,
                                          (2*intervals[i,1]-1) :
                                            (2*intervals[i,2]+1))
     }
-    abline(v=c(extended_grid[unlist(intervals[,"begin"])],
+    graphics::abline(v=c(extended_grid[unlist(intervals[,"begin"])],
                tail(extended_grid,1)),lty=2,col="gray60",lwd=1)
 
     # if(max(index) > length(grid))
     if(!is.null(index)){
       new_scaled_x[,-index] <- NA
-      matplot(new_grid,t(new_scaled_x),type="l",lty=1,col=bg_col,lwd=lwds+1,add=TRUE)
+      graphics::matplot(new_grid,t(new_scaled_x),type="l",lty=1,col=bg_col,lwd=lwds+1,add=TRUE)
     }
-    abline(h=0)
+    graphics::abline(h=0)
   }else{
     new_x <- matrix(NA,nrow=nrow(x),ncol=2*ncol(x)+1)
     new_x[,seq(2,ncol(new_x),by=2)] <- as.matrix(x)
     for(i in seq(3,ncol(new_x)-1,by=2))
       new_x[,i] <- 0.5*(new_x[,i-1]+new_x[,i+1])
 
-    matplot(grid,t(x) ,type="l",lty=1,col=cols,lwd=lwds+1,
+    graphics::matplot(grid,t(x) ,type="l",lty=1,col=cols,lwd=lwds+1,
             main="",xlab="",ylab="")
     for(i in 1:nrow(intervals)){
-      if(intervals[i,3]!=0) text( (extended_grid[intervals[i,1]] + grid[intervals[i,2]])/2 ,
+      if(intervals[i,3]!=0) graphics::text( (extended_grid[intervals[i,1]] + grid[intervals[i,2]])/2 ,
                                   max(x), round(intervals[i,3],1),cex=1)
       if(intervals[i,3] == 0) index <- c(index,
                                          (2*intervals[i,1]-1) :
                                            (2*intervals[i,2]+1))
     }
-    abline(v=c(extended_grid[unlist(intervals[,"begin"])],
+    graphics::abline(v=c(extended_grid[unlist(intervals[,"begin"])],
                tail(extended_grid,1)),lty=2,col="gray60",lwd=1)
 
     if(!is.null(index)){
       new_x[,-index] <- NA
-      matplot(new_grid,t(new_x),type="l",lty=1,col=bg_col,lwd=lwds+1,add=TRUE)
+      graphics::matplot(new_grid,t(new_x),type="l",lty=1,col=bg_col,lwd=lwds+1,add=TRUE)
     }
     x_center <- apply(x,2,mean)
-    lines(grid,x_center)
+    graphics::lines(grid,x_center)
   }
 }
 
@@ -309,33 +233,33 @@ interpretation_plot <- function(data,Bliss_estimate,q=1,centered=FALSE,cols=NULL
 #' # Compute the posterior density of the MCMC sample :
 #' res_poste <- dposterior(res_bliss1$posterior_sample,data1)
 dposterior <- function(posterior_sample,data,theta=NULL){
- if(!is.null(theta)){
-  if(is.null(dim(theta))){
-   rposterior <- as.matrix(t(theta))
+  if(!is.null(theta)){
+    if(is.null(dim(theta))){
+      rposterior <- as.matrix(t(theta))
+    }
+    K <- (ncol(theta)-2)/3
+  }else{
+    rposterior <- as.matrix(posterior_sample$trace)
+    K <- posterior_sample$param$K
   }
-  K <- (ncol(theta)-2)/3
- }else{
-  rposterior <- posterior_sample$trace
-  K <- posterior_sample$param$K
- }
- N <- nrow(rposterior)
- Q <- length(as.vector(K))
+  N <- nrow(rposterior)
+  Q <- length(as.vector(K))
 
- y <- data$y
- potential_intervals  <- posterior_sample$param$potential_intervals
- potential_intervals_dims <- list()
- for(q in 1:Q){
-  potential_intervals_dims[[q]] <- c(ncol(data$x[[q]]),
-                                     posterior_sample$param$l_values_length[[q]],
-                                     length(data$y))
- }
+  y <- data$y
+  potential_intervals  <- posterior_sample$param$potential_intervals
+  potential_intervals_dims <- list()
+  for(q in 1:Q){
+    potential_intervals_dims[[q]] <- c(ncol(data$x[[q]]),
+                                       posterior_sample$param$l_values_length[[q]],
+                                       length(data$y))
+  }
 
- res <- dposterior_cpp(rposterior,y,N,as.vector(K),potential_intervals,potential_intervals_dims,
-                       as.vector(posterior_sample$param$l_values_length),Q)
- colnames(res) <- c("posterior density","log posterior density",
-                    "likelihood","log likelihood",
-                    "prior density","log prior density")
- return(res)
+  res <- dposterior_cpp(rposterior,y,N,as.vector(K),potential_intervals,potential_intervals_dims,
+                        as.vector(posterior_sample$param$l_values_length),Q)
+  colnames(res) <- c("posterior density","log posterior density",
+                     "likelihood","log likelihood",
+                     "prior density","log prior density")
+  return(res)
 }
 
 ################################# ----
@@ -358,43 +282,16 @@ dposterior <- function(posterior_sample,data,theta=NULL){
 #' @useDynLib bliss
 #' @importFrom Rcpp sourceCpp
 #' @importFrom stats cor
-#' @export
 #' @examples
 #' \donttest{
-#' param_sim <- list(Q=1,
-#'                   n=100,
-#'                   p=c(50),
-#'                   grids_lim=list(c(0,1)))
-#' data <- sim(param_sim,verbose=TRUE)
-#'
-#' param <- list(iter=5e2,
-#'               K=c(3),
-#'               n_chains = 3)
-#' res_bliss <- fit_Bliss(data,param,verbose=TRUE,compute_density=FALSE,sann=FALSE)
-#'
-#' param$grids <- data$grids
-#' chains_info1 <- compute_chains_info(res_bliss$chains[[1]],param)
-#' chains_info2 <- compute_chains_info(res_bliss$chains[[2]],param)
-#' chains_info3 <- compute_chains_info(res_bliss$chains[[3]],param)
-#'
-#' # Smooth estimates
-#' ylim <- range(range(chains_info1$estimates$Smooth_estimate),
-#' range(chains_info2$estimates$Smooth_estimate),
-#' range(chains_info3$estimates$Smooth_estimate))
-#' plot(data$grids[[1]],chains_info1$estimates$Smooth_estimate,type="l",ylim=ylim,
-#' xlab="grid",ylab="")
-#' lines(data$grids[[1]],chains_info2$estimates$Smooth_estimate,col=2)
-#' lines(data$grids[[1]],chains_info3$estimates$Smooth_estimate,col=3)
-#'
-#' # Autocorrelation
-#' plot(chains_info1$autocorr_lag[,1],type="h")
+#' a=1
 #' }
 compute_chains_info <- function(chain,param){
   # Trace
   trace <- chain$trace
   # Beta sample
   Q <- length(param$K)
-  beta_sample <- compute_beta_sample(chain,param,Q)
+  beta_sample <- compute_beta_sample(chain,param)
 
   # Estimate mu beta sigma
   mu_hat <- mean(trace[,'mu'])
